@@ -20,27 +20,23 @@
  * cpu_number_map in other architectures.
  */
 
-#include <asm/sigp.h>
-#include <asm/timex.h>
-#include <asm/init.h>
-#include <asm/spinlock.h>
 #include <linux/module.h>
-#include <linux/stddef.h>
-#include <linux/kernel.h>
-#include <linux/kernel_stat.h>
-#include <linux/smp.h>
-#include <linux/mm.h>
-#include <asm/pgtable.h>
-#include <asm/string.h>
-#include <asm/s390_ext.h>
+#include <linux/init.h>
 
-#include "cpcmd.h"
+#include <linux/mm.h>
+#include <linux/spinlock.h>
+#include <linux/kernel_stat.h>
+#include <linux/smp_lock.h>
+
+#include <linux/delay.h>
+
+#include <asm/sigp.h>
+#include <asm/pgalloc.h>
 #include <asm/irq.h>
+#include <asm/s390_ext.h>
+#include <asm/cpcmd.h>
 
 /* prototypes */
-extern void update_one_process( struct task_struct *p,
-                                unsigned long ticks, unsigned long user,
-                                unsigned long system, int cpu);
 extern int cpu_idle(void * unused);
 
 extern __u16 boot_cpu_addr;
@@ -53,11 +49,10 @@ static int       max_cpus = NR_CPUS;	  /* Setup configured maximum number of CPU
 int              smp_num_cpus;
 struct _lowcore *lowcore_ptr[NR_CPUS];
 unsigned int     prof_multiplier[NR_CPUS];
+unsigned int     prof_old_multiplier[NR_CPUS];
 unsigned int     prof_counter[NR_CPUS];
-volatile int     cpu_number_map[NR_CPUS];
 cycles_t         cacheflush_time=0;
 int              smp_threads_ready=0;      /* Set when the idlers are all forked. */
-unsigned long    ipi_count=0;              /* Number of IPIs delivered. */
 static atomic_t  smp_commenced = ATOMIC_INIT(0);
 
 spinlock_t       kernel_flag = SPIN_LOCK_UNLOCKED;
@@ -73,31 +68,40 @@ spinlock_t       kernel_flag = SPIN_LOCK_UNLOCKED;
  *      SMP mode to <NUM>.
  */
 
-void __init smp_setup(char *str, int *ints)
+static int __init nosmp(char *str)
 {
-   if (ints && ints[0] > 0)
-      max_cpus = ints[1];
-   else
-      max_cpus = 0;
+	max_cpus = 0;
+	return 1;
 }
 
+__setup("nosmp", nosmp);
+
+static int __init maxcpus(char *str)
+{
+	get_option(&str, &max_cpus);
+	return 1;
+}
+
+__setup("maxcpus=", maxcpus);
+
 /*
  * Reboot, halt and power_off routines for SMP.
  */
 extern char vmhalt_cmd[];
 extern char vmpoff_cmd[];
-extern void reipl(int ipl_device);
+
+extern void reipl(unsigned long devno);
 
 void do_machine_restart(void)
 {
         smp_send_stop();
-	reipl(S390_lowcore.ipl_device); 
+	reipl(S390_lowcore.ipl_device);
 }
 
 void machine_restart(char * __unused) 
 {
         if (smp_processor_id() != 0) {
-                smp_ext_call_async(0, ec_restart);
+                smp_ext_bitcall(0, ec_restart);
                 for (;;);
         } else
                 do_machine_restart();
@@ -106,7 +110,7 @@ void machine_restart(char * __unused)
 void do_machine_halt(void)
 {
         smp_send_stop();
-        if (MACHINE_IS_VM && strlen(vmhalt_cmd) > 0) 
+        if (MACHINE_IS_VM && strlen(vmhalt_cmd) > 0)
                 cpcmd(vmhalt_cmd, NULL, 0);
         signal_processor(smp_processor_id(), sigp_stop_and_store_status);
 }
@@ -114,7 +118,7 @@ void do_machine_halt(void)
 void machine_halt(void)
 {
         if (smp_processor_id() != 0) {
-                smp_ext_call_async(0, ec_halt);
+                smp_ext_bitcall(0, ec_halt);
                 for (;;);
         } else
                 do_machine_halt();
@@ -131,7 +135,7 @@ void do_machine_power_off(void)
 void machine_power_off(void)
 {
         if (smp_processor_id() != 0) {
-                smp_ext_call_async(0, ec_power_off);
+                smp_ext_bitcall(0, ec_power_off);
                 for (;;);
         } else
                 do_machine_power_off();
@@ -142,7 +146,7 @@ void machine_power_off(void)
  * cpus are handled.
  */
 
-void do_ext_call_interrupt(struct pt_regs *regs, __u16 source_cpu_addr)
+void do_ext_call_interrupt(struct pt_regs *regs, __u16 code)
 {
         ec_ext_call *ec, *next;
         int bits;
@@ -165,6 +169,8 @@ void do_ext_call_interrupt(struct pt_regs *regs, __u16 source_cpu_addr)
 		do_machine_halt();
         if (test_bit(ec_power_off, &bits))
 		do_machine_power_off();
+	if (test_bit(ec_ptlb, &bits))
+	        local_flush_tlb();
 
         /*
          * Handle external call commands with a parameter area
@@ -189,74 +195,21 @@ void do_ext_call_interrupt(struct pt_regs *regs, __u16 source_cpu_addr)
         /* Execute every sigp command on the queue */
         while (ec != NULL) {
                 switch (ec->cmd) {
-                case ec_get_ctl: {
-                        ec_creg_parms *pp;
-                        pp = (ec_creg_parms *) ec->parms;
-                        atomic_set(&ec->status,ec_executing);
-                        asm volatile (
-                                "   bras  1,0f\n"
-                                "   stctl 0,0,0(%0)\n"
-                                "0: ex    %1,0(1)\n"
-                                : : "a" (pp->cregs+pp->start_ctl),
-                                "a" ((pp->start_ctl<<4) + pp->end_ctl)
-                                : "memory", "1" );
-                        atomic_set(&ec->status,ec_done);
-                        return;
-                }
-                case ec_set_ctl: {
-                        ec_creg_parms *pp;
-                        pp = (ec_creg_parms *) ec->parms;
-                        atomic_set(&ec->status,ec_executing);
-                        asm volatile (
-                                "   bras  1,0f\n"
-                                "   lctl 0,0,0(%0)\n"
-                                "0: ex    %1,0(1)\n"
-                                : : "a" (pp->cregs+pp->start_ctl),
-                                "a" ((pp->start_ctl<<4) + pp->end_ctl)
-                                : "memory", "1" );
-                        atomic_set(&ec->status,ec_done);
-                        return;
-                }
-                case ec_set_ctl_masked: {
-                        ec_creg_mask_parms *pp;
-                        u32 cregs[16];
-                        int i;
+                case ec_callback_async: {
+                        void (*func)(void *info);
+                        void *info;
 
-                        pp = (ec_creg_mask_parms *) ec->parms;
+                        func = ec->func;
+                        info = ec->info;
                         atomic_set(&ec->status,ec_executing);
-                        asm volatile (
-                                "   bras  1,0f\n"
-                                "   stctl 0,0,0(%0)\n"
-                                "0: ex    %1,0(1)\n"
-                                : : "a" (cregs+pp->start_ctl),
-                                "a" ((pp->start_ctl<<4) + pp->end_ctl)
-                                : "memory", "1" );
-                        for (i = pp->start_ctl; i <= pp->end_ctl; i++)
-                                cregs[i] = (cregs[i] & pp->andvals[i])
-                                                     | pp->orvals[i];
-                        asm volatile (
-                                "   bras  1,0f\n"
-                                "   lctl 0,0,0(%0)\n"
-                                "0: ex    %1,0(1)\n"
-                                : : "a" (cregs+pp->start_ctl),
-                                "a" ((pp->start_ctl<<4) + pp->end_ctl)
-                                : "memory", "1" );
-                        atomic_set(&ec->status,ec_done);
+                        (func)(info);
                         return;
                 }
-		case ec_ptlb:
-			atomic_set(&ec->status, ec_executing);
-			__flush_tlb();
-			atomic_set(&ec->status, ec_done);
-		        return;
-                case ec_callback: {
-                        ec_callback_parms *cbp;
-                        cbp = (ec_callback_parms *) ec->parms;
+                case ec_callback_sync:
                         atomic_set(&ec->status,ec_executing);
-                        (cbp->callback)(cbp->data);
+                        (ec->func)(ec->info);
                         atomic_set(&ec->status,ec_done);
                         return;
-                }
                 default:
                 }
                 ec = ec->next;
@@ -264,17 +217,19 @@ void do_ext_call_interrupt(struct pt_regs *regs, __u16 source_cpu_addr)
 }
 
 /*
- * Send an external call sigp to another cpu and wait for its completion.
+ * Send a callback sigp to another cpu.
  */
-sigp_ccode smp_ext_call_sync(int cpu, ec_cmd_sig cmd, void *parms)
+sigp_ccode
+smp_ext_call(int cpu, void (*func)(void *info), void *info, int wait)
 {
         struct _lowcore *lowcore = &get_cpu_lowcore(cpu);
         sigp_ccode ccode;
         ec_ext_call ec;
 
-        ec.cmd = cmd;
+        ec.cmd = wait ? ec_callback_sync : ec_callback_async;
         atomic_set(&ec.status, ec_pending);
-        ec.parms = parms;
+	ec.func = func;
+        ec.info = info;
         do {
                 ec.next = (ec_ext_call*) atomic_read(&lowcore->ext_call_queue);
         } while (atomic_compare_and_swap((int) ec.next, (int)(&ec),
@@ -294,17 +249,51 @@ sigp_ccode smp_ext_call_sync(int cpu, ec_cmd_sig cmd, void *parms)
 
         if (ccode != sigp_not_operational)
                 /* wait for completion, FIXME: possible seed of a deadlock */
-                while (atomic_read(&ec.status) != ec_done);
+                while (atomic_read(&ec.status) != (wait?ec_done:ec_executing));
 
         return ccode;
 }
 
 /*
- * Send an external call sigp to another cpu and return without waiting
- * for its completion. Currently we do not support parameters with
- * asynchronous sigps.
+ * Send a callback sigp to every other cpu in the system.
  */
-sigp_ccode smp_ext_call_async(int cpu, ec_bit_sig sig)
+void smp_ext_call_others(void (*func)(void *info), void *info, int wait)
+{
+        struct _lowcore *lowcore;
+        ec_ext_call ec[NR_CPUS];
+        sigp_ccode ccode;
+        int i;
+
+        for (i = 0; i < smp_num_cpus; i++) {
+                if (smp_processor_id() == i)
+                        continue;
+                lowcore = &get_cpu_lowcore(i);
+		ec[i].cmd = wait ? ec_callback_sync : ec_callback_async;
+		atomic_set(&ec[i].status, ec_pending);
+		ec[i].func = func;
+		ec[i].info = info;
+                do {
+                        ec[i].next = (ec_ext_call *)
+                                        atomic_read(&lowcore->ext_call_queue);
+                } while (atomic_compare_and_swap((int) ec[i].next, (int)(ec+i),
+                                                 &lowcore->ext_call_queue));
+                ccode = signal_processor(i, sigp_external_call);
+        }
+
+        /* wait for completion, FIXME: possible seed of a deadlock */
+        for (i = 0; i < smp_num_cpus; i++) {
+                if (smp_processor_id() == i)
+                        continue;
+                while (atomic_read(&ec[i].status) != 
+		       (wait ? ec_done:ec_executing));
+        }
+}
+
+/*
+ * Send an external call sigp to another cpu and return without waiting
+ * for its completion.
+ */
+sigp_ccode smp_ext_bitcall(int cpu, ec_bit_sig sig)
 {
         struct _lowcore *lowcore = &get_cpu_lowcore(cpu);
         sigp_ccode ccode;
@@ -319,44 +308,9 @@ sigp_ccode smp_ext_call_async(int cpu, ec_bit_sig sig)
 
 /*
  * Send an external call sigp to every other cpu in the system and
- * wait for the completion of the sigps.
+ * return without waiting for its completion.
  */
-void smp_ext_call_sync_others(ec_cmd_sig cmd, void *parms)
-{
-        struct _lowcore *lowcore;
-        ec_ext_call ec[NR_CPUS];
-        sigp_ccode ccode;
-        int i;
-
-        for (i = 0; i < smp_num_cpus; i++) {
-                if (smp_processor_id() == i)
-                        continue;
-                lowcore = &get_cpu_lowcore(i);
-                ec[i].cmd = cmd;
-                atomic_set(&ec[i].status, ec_pending);
-                ec[i].parms = parms;
-                do {
-                        ec[i].next = (ec_ext_call *)
-                                        atomic_read(&lowcore->ext_call_queue);
-                } while (atomic_compare_and_swap((int) ec[i].next, (int)(ec+i),
-                                                 &lowcore->ext_call_queue));
-                ccode = signal_processor(i, sigp_external_call);
-        }
-
-        /* wait for completion, FIXME: possible seed of a deadlock */
-        for (i = 0; i < smp_num_cpus; i++) {
-                if (smp_processor_id() == i)
-                        continue;
-                while (atomic_read(&ec[i].status) != ec_done);
-        }
-}
-
-/*
- * Send an external call sigp to every other cpu in the system and
- * return without waiting for the completion of the sigps. Currently
- * we do not support parameters with asynchronous sigps.
- */
-void smp_ext_call_async_others(ec_bit_sig sig)
+void smp_ext_bitcall_others(ec_bit_sig sig)
 {
         struct _lowcore *lowcore;
         sigp_ccode ccode;
@@ -384,12 +338,12 @@ void smp_ext_call_async_others(ec_bit_sig sig)
  * do
  * {
  *    info->cpu=next_cpu;
- *    next_cpu=smp_signal_others(order_code,parameter,TRUE,info);
+ *    next_cpu=smp_signal_others(order_code,parameter,1,info);
  *    ... check info here
  * } while(next_cpu<=smp_num_cpus)
  *
  *  if you are lazy just use it like
- * smp_signal_others(order_code,parameter,0,TRUE,NULL);
+ * smp_signal_others(order_code,parameter,0,1,NULL);
  */
 int smp_signal_others(sigp_order_code order_code, u32 parameter,
                       int spin, sigp_info *info)
@@ -399,7 +353,7 @@ int smp_signal_others(sigp_order_code order_code, u32 parameter,
         u16          i;
 
         if (info)
-                info->intresting = FALSE;
+                info->intresting = 0;
         for (i = (info ? info->cpu : 0); i < smp_num_cpus; i++) {
                 if (smp_processor_id() != i) {
                         do {
@@ -408,7 +362,7 @@ int smp_signal_others(sigp_order_code order_code, u32 parameter,
                                         parameter, i, order_code);
                         } while(spin && ccode == sigp_busy);
                         if (info && ccode != sigp_order_code_accepted) {
-                                info->intresting = TRUE;
+                                info->intresting = 1;
                                 info->cpu = i;
                                 info->ccode = ccode;
                                 i++;
@@ -426,7 +380,47 @@ int smp_signal_others(sigp_order_code order_code, u32 parameter,
 
 void smp_send_stop(void)
 {
-        smp_signal_others(sigp_stop_and_store_status, 0, TRUE, NULL);
+        int i;
+        u32 dummy;
+        unsigned long low_core_addr;
+
+        /* write magic number to zero page (absolute 0) */
+
+        get_cpu_lowcore(smp_processor_id()).panic_magic = __PANIC_MAGIC;
+
+        /* stop all processors */
+
+        smp_signal_others(sigp_stop, 0, 1, NULL);
+
+        /* store status of all processors in their lowcores (real 0) */
+
+        for (i =  0; i < smp_num_cpus; i++) {
+                if (smp_processor_id() != i) {
+                        int ccode;
+                        low_core_addr = (unsigned long)&get_cpu_lowcore(i);
+                        do {
+                                ccode = signal_processor_ps(
+                                   &dummy,
+                                   low_core_addr,
+                                   i,
+                                   sigp_store_status_at_address);
+                        } while(ccode == sigp_busy);
+                }
+        }
+}
+
+/*
+ * this function sends a 'purge tlb' signal to another CPU.
+ */
+void smp_ptlb_callback(void *info)
+{
+	local_flush_tlb();
+}
+
+void smp_ptlb_all(void)
+{
+        smp_ext_call_others(smp_ptlb_callback, NULL, 1);
+	local_flush_tlb();
 }
 
 /*
@@ -437,7 +431,44 @@ void smp_send_stop(void)
 
 void smp_send_reschedule(int cpu)
 {
-        smp_ext_call_async(cpu, ec_schedule);
+        smp_ext_bitcall(cpu, ec_schedule);
+}
+
+/*
+ * parameter area for the set/clear control bit callbacks
+ */
+typedef struct
+{
+	__u16 start_ctl;
+	__u16 end_ctl;
+	__u32 orvals[16];
+	__u32 andvals[16];
+} ec_creg_mask_parms;
+
+/*
+ * callback for setting/clearing control bits
+ */
+void smp_ctl_bit_callback(void *info) {
+	ec_creg_mask_parms *pp;
+	u32 cregs[16];
+	int i;
+	
+	pp = (ec_creg_mask_parms *) info;
+	asm volatile ("   bras  1,0f\n"
+		      "   stctl 0,0,0(%0)\n"
+		      "0: ex    %1,0(1)\n"
+		      : : "a" (cregs+pp->start_ctl),
+		          "a" ((pp->start_ctl<<4) + pp->end_ctl)
+		      : "memory", "1" );
+	for (i = pp->start_ctl; i <= pp->end_ctl; i++)
+		cregs[i] = (cregs[i] & pp->andvals[i]) | pp->orvals[i];
+	asm volatile ("   bras  1,0f\n"
+		      "   lctl 0,0,0(%0)\n"
+		      "0: ex    %1,0(1)\n"
+		      : : "a" (cregs+pp->start_ctl),
+		          "a" ((pp->start_ctl<<4) + pp->end_ctl)
+		      : "memory", "1" );
+	return;
 }
 
 /*
@@ -451,7 +482,7 @@ void smp_ctl_set_bit(int cr, int bit) {
                 parms.end_ctl = cr;
                 parms.orvals[cr] = 1 << bit;
                 parms.andvals[cr] = 0xFFFFFFFF;
-                smp_ext_call_sync_others(ec_set_ctl_masked,&parms);
+                smp_ext_call_others(smp_ctl_bit_callback, &parms, 1);
         }
         __ctl_set_bit(cr, bit);
 }
@@ -467,23 +498,33 @@ void smp_ctl_clear_bit(int cr, int bit) {
                 parms.end_ctl = cr;
                 parms.orvals[cr] = 0x00000000;
                 parms.andvals[cr] = ~(1 << bit);
-                smp_ext_call_sync_others(ec_set_ctl_masked,&parms);
+                smp_ext_call_others(smp_ctl_bit_callback, &parms, 1);
         }
         __ctl_clear_bit(cr, bit);
 }
 
 /*
- * Execute a callback function on all cpus
+ * Call a function on all other processors
  */
-void smp_do_callback_all(void (*callback)(void *), void *data) {
-        ec_callback_parms parms;
 
-        if (atomic_read(&smp_commenced) != 0) {
-                parms.callback = callback;
-                parms.data = data;
-                smp_ext_call_sync_others(ec_callback, &parms);
-        }
-        (callback)(data);
+int
+smp_call_function(void (*func)(void *info), void *info, int retry, int wait)
+/*
+ * [SUMMARY] Run a function on all other CPUs.
+ * <func> The function to run. This must be fast and non-blocking.
+ * <info> An arbitrary pointer to pass to the function.
+ * <retry> currently unused.
+ * <wait> If true, wait (atomically) until function has completed on other CPUs.
+ * [RETURNS] 0 on success, else a negative status code. Does not return until
+ * remote CPUs are nearly ready to execute <<func>> or are or have executed.
+ *
+ * You must not call this function with disabled interrupts or from a
+ * hardware interrupt handler, you may call it from a bottom half handler.
+ */
+{
+        if (atomic_read(&smp_commenced) != 0)
+                smp_ext_call_others(func, info, wait);
+        return 0;
 }
 
 /*
@@ -515,7 +556,7 @@ void smp_count_cpus(void)
  *      Activate a secondary processor.
  */
 extern void init_100hz_timer(void);
-extern void cpu_init (void);
+extern int pfault_token(void);
 
 int __init start_secondary(void *cpuvoid)
 {
@@ -528,6 +569,10 @@ int __init start_secondary(void *cpuvoid)
                 /* nothing */ ;
         /* init per CPU 100 hz timer */
         init_100hz_timer();
+#ifdef CONFIG_PFAULT
+	/* Enable pfault pseudo page faults on this cpu. */
+	pfault_init();
+#endif
         /* cpu_idle will call schedule for us */
         return cpu_idle(NULL);
 }
@@ -541,28 +586,47 @@ void __init initialize_secondary(void)
 {
 }
 
+static int __init fork_by_hand(void)
+{
+       struct pt_regs regs;
+       /* don't care about the psw and regs settings since we'll never
+          reschedule the forked task. */
+       memset(&regs,0,sizeof(struct pt_regs));
+       return do_fork(CLONE_VM|CLONE_PID, 0, &regs, 0);
+}
+
 static void __init do_boot_cpu(int cpu)
 {
         struct task_struct *idle;
         struct _lowcore    *cpu_lowcore;
+
+        /* We can't use kernel_thread since we must _avoid_ to reschedule
+           the child. */
+        if (fork_by_hand() < 0)
+                panic("failed fork for CPU %d", cpu);
+
         /*
-         *	We need an idle process for each processor.
+         * We remove it from the pidhash and the runqueue
+         * once we got the process:
          */
-        
-        kernel_thread(start_secondary,(void *)cpu, CLONE_PID);
-        idle = task[cpu];
+        idle = init_task.prev_task;
         if (!idle)
                 panic("No idle process for CPU %d",cpu);
         idle->processor = cpu;
-        cpu_number_map[cpu] = cpu;
+        idle->has_cpu = 1; /* we schedule the first task manually */
+
+        del_from_runqueue(idle);
+        unhash_process(idle);
+        init_tasks[cpu] = idle;
+
         cpu_lowcore=&get_cpu_lowcore(cpu);
-        cpu_lowcore->kernel_stack=idle->tss.ksp;
+        cpu_lowcore->kernel_stack=idle->thread.ksp;
         __asm__ __volatile__("stctl 0,15,%0\n\t"
                              "stam  0,15,%1"
-                             : "=m" (cpu_lowcore->cregs_save_area[0]), 
+                             : "=m" (cpu_lowcore->cregs_save_area[0]),
                                "=m" (cpu_lowcore->access_regs_save_area[0])
                              : : "memory");
-        
+
         eieio();
         signal_processor(cpu,sigp_restart);
 }
@@ -593,7 +657,6 @@ void __init smp_boot_cpus(void)
 {
         struct _lowcore *curr_lowcore;
         sigp_ccode   ccode;
-        int curr_cpu;
         int i;
 
         /* request the 0x1202 external interrupt */
@@ -608,42 +671,41 @@ void __init smp_boot_cpus(void)
          */
         
         for (i = 0; i < NR_CPUS; i++) {
-                cpu_number_map[i] = -1;
                 prof_counter[i] = 1;
+                prof_old_multiplier[i] = 1;
                 prof_multiplier[i] = 1;
         }
 
-        cpu_number_map[0] = 0;
-	print_cpu_info(&safe_get_cpu_lowcore(0).cpu_data);
+        print_cpu_info(&safe_get_cpu_lowcore(0).cpu_data);
 
-        for(curr_cpu = 0; curr_cpu < smp_num_cpus; curr_cpu++)
+        for(i = 0; i < smp_num_cpus; i++)
         {
                 curr_lowcore = (struct _lowcore *)
                                     __get_free_page(GFP_KERNEL|GFP_DMA);
                 if (curr_lowcore == NULL) {
-                        printk("smp_boot_cpus failed to allocate prefix memory\n"); 
+                        printk("smp_boot_cpus failed to allocate prefix memory\n");
                         break;
                 }
-                lowcore_ptr[curr_cpu] = curr_lowcore;
+                lowcore_ptr[i] = curr_lowcore;
                 memcpy(curr_lowcore, &S390_lowcore, sizeof(struct _lowcore));
                 /*
                  * Most of the parameters are set up when the cpu is
                  * started up.
                  */
-                if(smp_processor_id()==curr_cpu)
-                        set_prefix((u32)curr_lowcore);
+                if (smp_processor_id() == i)
+                        set_prefix((u32) curr_lowcore);
                 else {
-                        ccode=signal_processor_p((u32)(curr_lowcore),
-                                                 curr_cpu,sigp_set_prefix);
+                        ccode = signal_processor_p((u32)(curr_lowcore),
+                                                   i, sigp_set_prefix);
                         if(ccode) {
-                                /* if this gets troublesome I'll have to do 
+                                /* if this gets troublesome I'll have to do
                                  * something about it. */
                                 printk("ccode %d for cpu %d  returned when "
                                        "setting prefix in smp_boot_cpus not good.\n",
-                                       (int)ccode,(int)curr_cpu);
+                                       (int) ccode, (int) i);
                         }
                         else
-                                do_boot_cpu(curr_cpu);
+                                do_boot_cpu(i);
                 }
         }
 }
@@ -671,6 +733,7 @@ int setup_profiling_timer(unsigned int multiplier)
 
 void smp_local_timer_interrupt(struct pt_regs * regs)
 {
+	int user = (user_mode(regs) != 0);
         int cpu = smp_processor_id();
 
         /*
@@ -683,9 +746,24 @@ void smp_local_timer_interrupt(struct pt_regs * regs)
                 s390_do_profile(regs->psw.addr);
 
         if (!--prof_counter[cpu]) {
-                int user=0,system=0;
+                int system = 1-user;
                 struct task_struct * p = current;
 
+                /*
+                 * The multiplier may have changed since the last time we got
+                 * to this point as a result of the user writing to
+                 * /proc/profile.  In this case we need to adjust the APIC
+                 * timer accordingly.
+                 *
+                 * Interrupts are already masked off at this point.
+                 */
+                prof_counter[cpu] = prof_multiplier[cpu];
+                if (prof_counter[cpu] != prof_old_multiplier[cpu]) {
+			/* FIXME setup_APIC_timer(calibration_result/prof_counter[cpu]
+			   ); */
+                  prof_old_multiplier[cpu] = prof_counter[cpu];
+                }
+
                 /*
                  * After doing the above, we need to make like
                  * a normal interrupt - otherwise timer interrupts
@@ -693,36 +771,14 @@ void smp_local_timer_interrupt(struct pt_regs * regs)
                  * WrongThing (tm) to do.
                  */
 
-                if (user_mode(regs))
-                        user=1;
-                else
-                        system=1;
-
                 irq_enter(cpu, 0);
-                update_one_process(p, 1, user, system, cpu);
-                if (p->pid) {
-                        p->counter -= 1;
-                        if (p->counter < 0) {
-                                p->counter = 0;
-                                p->need_resched = 1;
-                        }
-                        if (p->priority < DEF_PRIORITY) {
-                                kstat.cpu_nice += user;
-                                kstat.per_cpu_nice[cpu] += user;
-                        } else {
-                                kstat.cpu_user += user;
-                                kstat.per_cpu_user[cpu] += user;
-                        }
-                        kstat.cpu_system += system;
-                        kstat.per_cpu_system[cpu] += system;
-
-                }
-                prof_counter[cpu]=prof_multiplier[cpu];
+		update_process_times(user);
                 irq_exit(cpu, 0);
         }
 }
 
+EXPORT_SYMBOL(lowcore_ptr);
 EXPORT_SYMBOL(kernel_flag);
 EXPORT_SYMBOL(smp_ctl_set_bit);
 EXPORT_SYMBOL(smp_ctl_clear_bit);
-EXPORT_SYMBOL(smp_do_callback_all);
+EXPORT_SYMBOL(smp_num_cpus);
