@@ -29,7 +29,7 @@ static char *version =
 #include <linux/ptrace.h>
 #include <linux/ioport.h>
 #include <linux/in.h>
-#include <linux/malloc.h>
+#include <linux/slab.h>
 #include <linux/string.h>
 #include <linux/init.h>
 #include <asm/bitops.h>
@@ -205,7 +205,7 @@ static void inline soc_unsolicited (struct soc *s)
 		hwrsp = (soc_rsp *)sw_cq->pool + sw_cq->out;
 
 		hwrspc = NULL;
-		flags = hwrsp->shdr.flags;
+		flags = xram_get_16 ((xram_p)&hwrsp->shdr.flags);
 		count = xram_get_8 ((xram_p)&hwrsp->count);
 		fc = (fc_channel *)&s->port[flags & SOC_PORT_B];
 		SOD(("FC %08lx fcp_state_change %08lx\n",
@@ -565,14 +565,10 @@ static inline void soc_init(struct sbus_dev *sdev, int no)
 	     (long)socs, (long)soc_intr, (long)soc_hw_enque))	
 	if (version_printed++ == 0)
 		printk (version);
-#ifdef MODULE
-	s->port[0].fc.module = &__this_module;
-	s->port[1].fc.module = &__this_module;
-#else
-	s->port[0].fc.module = NULL;
-	s->port[1].fc.module = NULL;
-#endif
-	                                	
+
+	s->port[0].fc.module = THIS_MODULE;
+	s->port[1].fc.module = THIS_MODULE;
+
 	s->next = socs;
 	socs = s;
 	s->port[0].fc.dev = sdev;
@@ -668,7 +664,7 @@ static inline void soc_init(struct sbus_dev *sdev, int no)
 	memset (cq, 0, sizeof(cq));
 	
 	size = (SOC_CQ_REQ0_SIZE + SOC_CQ_REQ1_SIZE) * sizeof(soc_req);
-	s->req_cpu = sbus_alloc_consistant(sdev, size, &s->req_dvma);
+	s->req_cpu = sbus_alloc_consistent(sdev, size, &s->req_dvma);
 	s->req[0].pool = s->req_cpu;
 	cq[0].address = s->req_dvma;
 	s->req[1].pool = s->req[0].pool + SOC_CQ_REQ0_SIZE;
@@ -713,11 +709,7 @@ static inline void soc_init(struct sbus_dev *sdev, int no)
 	SOD(("Enabled SOC\n"))
 }
 
-#ifndef MODULE
-int __init soc_probe(void)
-#else
-int init_module(void)
-#endif
+static int __init soc_probe(void)
 {
 	struct sbus_bus *sbus;
 	struct sbus_dev *sdev = 0;
@@ -741,10 +733,7 @@ int init_module(void)
 	return 0;
 }
 
-EXPORT_NO_SYMBOLS;
-
-#ifdef MODULE
-void cleanup_module(void)
+static void __exit soc_cleanup(void)
 {
 	struct soc *s;
 	int irq;
@@ -765,9 +754,13 @@ void cleanup_module(void)
 			sbus_iounmap(s->xram, sdev->reg_addrs[1].reg_size);
 			sbus_iounmap(s->regs, sdev->reg_addrs[2].reg_size);
 		}
-		sbus_free_consistant(sdev,
+		sbus_free_consistent(sdev,
 				     (SOC_CQ_REQ0_SIZE+SOC_CQ_REQ1_SIZE)*sizeof(soc_req),
 				     s->req_cpu, s->req_dvma);
 	}
 }
-#endif
+
+EXPORT_NO_SYMBOLS;
+
+module_init(soc_probe);
+module_exit(soc_cleanup);
